// Code generated by go-swagger; DO NOT EDIT.

package volume

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// DiskUsage represents system data usage for volume resources.
//
// swagger:model DiskUsage
type DiskUsage struct {

	// Count of active volumes.
	//
	// Example: 1
	ActiveCount int64 `json:"ActiveCount,omitempty"`

	// List of volumes.
	//
	Items []Volume `json:"Items,omitempty"`

	// Disk space that can be reclaimed by removing inactive volumes.
	//
	// Example: 12345678
	Reclaimable int64 `json:"Reclaimable,omitempty"`

	// Count of all volumes.
	//
	// Example: 4
	TotalCount int64 `json:"TotalCount,omitempty"`

	// Disk space in use by volumes.
	//
	// Example: 98765432
	TotalSize int64 `json:"TotalSize,omitempty"`
}
