// Code generated by go-swagger; DO NOT EDIT.

package network

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"net/netip"
)

// Task carries the information about one backend task
//
// swagger:model Task
type Task struct {

	// name
	Name string `json:"Name"`

	// endpoint ID
	EndpointID string `json:"EndpointID"`

	// endpoint IP
	EndpointIP netip.Addr `json:"EndpointIP"`

	// info
	Info map[string]string `json:"Info"`
}
