// Copyright The OpenTelemetry Authors
// SPDX-License-Identifier: Apache-2.0

package otlptrace // import "go.opentelemetry.io/otel/exporters/otlp/otlptrace"

// Version is the current release version of the OpenTelemetry OTLP trace exporter in use.
func Version() string {
	return "1.38.0"
}
